@@ -4,6 +4,7 @@ import shutil
 import logging
 
 import numpy as np
+import scipy.stats as stats
 import matplotlib.pyplot as plt
 
 from typing import *
@@ -568,18 +569,22 @@ class EnsemblePattern(PatternBase):
         return getattr(self, f"_{self._ensemble_method}_collate")
 
     # Core
+    
+    @staticmethod
+    def vote(arr: np.ndarray, num_classes: int) -> np.ndarray:
+        counts = np.apply_along_axis(partial(np.bincount, minlength=num_classes), 1, arr)
+        return counts.argmax(1).reshape([-1, 1])
 
     @staticmethod
     def _default_collate(arrays: List[np.ndarray],
                          requires_prob: bool) -> np.ndarray:
         predictions = np.array(arrays)
         if not requires_prob and np.issubdtype(predictions.dtype, np.integer):
-            max_class = predictions.max() + 1
+            num_classes = predictions.max() + 1
             if len(predictions.shape) == 3:
                 predictions = predictions.squeeze(2)
             predictions = predictions.T
-            counts = np.apply_along_axis(partial(np.bincount, minlength=max_class), 1, predictions)
-            return counts.argmax(1).reshape([-1, 1])
+            return EnsemblePattern.vote(predictions, num_classes)
         return predictions.mean(0)
 
     # API
@@ -954,20 +959,28 @@ class Visualizer:
             tmp_save_name = f"{save_name}_{counter}"
         return tmp_save_name
 
-    def bar(self, data: np.ndarray, classes: list, categories: list, save_name="bar_plot", title="",
-            padding=1e-3, expand_floor=5, overwrite=True):
-        n_class, n_categories = map(len, [classes, categories])
-        data = [data / data.sum() + padding] if n_class == 1 else data / data.sum(0) + padding
-        expand_floor = expand_floor * 2 if n_class == 1 else expand_floor
-        colors = plt.cm.Paired([i / n_class for i in range(n_class)])
-        x_base = np.arange(1, n_categories + 1)
-        expand_ratio = max(1., n_categories / expand_floor)
+    def bar(self,
+            data: np.ndarray,
+            classes: List[Union[str, Any]],
+            categories: List[Union[str, Any]],
+            *,
+            title: str = "",
+            save_name: str = "bar_plot",
+            padding: float = 1e-3,
+            expand_floor: int = 5,
+            overwrite: bool = True):
+        num_classes, num_categories = map(len, [classes, categories])
+        data = [data / data.sum() + padding] if num_classes == 1 else data / data.sum(0) + padding
+        expand_floor = expand_floor * 2 if num_classes == 1 else expand_floor
+        colors = plt.cm.Paired([i / num_classes for i in range(num_classes)])
+        x_base = np.arange(1, num_categories + 1)
+        expand_ratio = max(1., num_categories / expand_floor)
         fig = plt.figure(figsize=(6.4 * expand_ratio, 4.8))
         plt.title(title)
-        n_divide = n_class - 1
+        n_divide = num_classes - 1
         width = 0.35 / max(1, n_divide)
-        cls_ratio = 0.5 if n_class == 1 else 1
-        for cls in range(n_class):
+        cls_ratio = 0.5 if num_classes == 1 else 1
+        for cls in range(num_classes):
             plt.bar(x_base - width * (0.5 * n_divide - cls_ratio * cls), data[cls], width=width,
                     facecolor=colors[cls], edgecolor="white",
                     label=classes[cls])
@@ -984,23 +997,31 @@ class Visualizer:
         plt.savefig(os.path.join(self.export_folder, f"{save_name}.png"))
         plt.close()
 
-    def function(self, f: Callable[[np.ndarray], np.ndarray], x_min: np.ndarray, x_max: np.ndarray,
-                 classes: list, categories: list, save_names=None, n_sample=1000,
-                 expand_floor=5, overwrite=True):
-        n_class, n_categories = map(len, [classes, categories])
+    def function(self,
+                 f: Callable[[np.ndarray], np.ndarray],
+                 x_min: np.ndarray,
+                 x_max: np.ndarray,
+                 classes: List[Union[str, Any]],
+                 categories: List[Union[str, Any]],
+                 *,
+                 save_names: List[str] = None,
+                 num_sample: int = 1000,
+                 expand_floor: int = 5,
+                 overwrite: bool = True):
+        num_classes, num_categories = map(len, [classes, categories])
         gaps = x_max - x_min
-        x_base = np.linspace(x_min - 0.1 * gaps, x_max + 0.1 * gaps, n_sample)
-        f_values = np.split(f(x_base), n_class, axis=1)
+        x_base = np.linspace(x_min - 0.1 * gaps, x_max + 0.1 * gaps, num_sample)
+        f_values = np.split(f(x_base), num_classes, axis=1)
         if save_names is None:
-            save_names = ["function_plot"] * n_categories
-        colors = plt.cm.Paired([i / n_class for i in range(n_class)])
+            save_names = ["function_plot"] * num_categories
+        colors = plt.cm.Paired([i / num_classes for i in range(num_classes)])
         expand_ratios = np.maximum(1., np.abs(x_min) / expand_floor, x_max / expand_floor)
         for i, (category, save_name, ratio, local_min, local_max, gap) in enumerate(zip(
                 categories, save_names, expand_ratios, x_min, x_max, gaps)):
             plt.figure(figsize=(6.4 * ratio, 4.8))
             plt.title(f"pdf for {category}")
             local_base = x_base[..., i]
-            for c in range(n_class):
+            for c in range(num_classes):
                 f_value = f_values[c][..., i].ravel()
                 plt.plot(local_base, f_value, c=colors[c], label=f"class: {classes[c]}")
             plt.xlim(local_min - 0.2 * gap, local_max + 0.2 * gap)
@@ -1347,10 +1368,125 @@ class Tracker:
         return trackers
 
 
+class DataInspector:
+    def __init__(self, data: generic_data_type):
+        self._data = np.asarray(data, np.float32)
+        self._sorted_data = np.sort(self._data)
+        self._num_samples = len(self._data)
+        self._mean = self._variance = self._std = None
+        self._moments = []
+        self._q1 = self._q3 = self._median = None
+
+    def get_moment(self, k: int) -> float:
+        if len(self._moments) < k:
+            self._moments += [None] * (k - len(self._moments))
+        if self._moments[k-1] is None:
+            self._moments[k-1] = np.sum((self._data - self.mean) ** k) / self._num_samples
+        return self._moments[k-1]
+
+    def get_quantile(self, q: float) -> float:
+        if not 0. <= q <= 1.:
+            raise ValueError("`q` should be in [0, 1]")
+        anchor = self._num_samples * q
+        int_anchor = int(anchor)
+        if not int(anchor % 1):
+            return self._sorted_data[int_anchor]
+        return 0.5 * (self._sorted_data[int_anchor-1] + self._sorted_data[int_anchor])
+
+    @property
+    def min(self) -> float:
+        return self._sorted_data[0]
+
+    @property
+    def max(self) -> float:
+        return self._sorted_data[-1]
+
+    @property
+    def mean(self) -> float:
+        if self._mean is None:
+            self._mean = self._data.mean()
+        return self._mean
+
+    @property
+    def variance(self) -> float:
+        if self._variance is None:
+            self._variance = np.sum((self._data - self.mean) ** 2) / (self._num_samples - 1)
+        return self._variance
+
+    @property
+    def std(self) -> float:
+        if self._std is None:
+            self._std = self.variance ** 0.5
+        return self._std
+
+    @property
+    def skewness(self) -> float:
+        n, moment3 = self._num_samples, self.get_moment(3)
+        return n ** 2 * moment3 / ((n - 1) * (n - 2) * self.std ** 3)
+
+    @property
+    def kurtosis(self) -> float:
+        n, moment4 = self._num_samples, self.get_moment(4)
+        return n**2*(n+1)*moment4 / ((n-1)*(n-2)*(n-3)*self.std**4) - 3*(n-1)**2/((n-2)*(n-3))
+
+    @property
+    def median(self) -> float:
+        if self._median is None:
+            self._median = self.get_quantile(0.5)
+        return self._median
+
+    @property
+    def q1(self) -> float:
+        if self._q1 is None:
+            self._q1 = self.get_quantile(0.25)
+        return self._q1
+
+    @property
+    def q3(self) -> float:
+        if self._q3 is None:
+            self._q3 = self.get_quantile(0.75)
+        return self._q3
+
+    @property
+    def range(self) -> float:
+        return self._sorted_data[-1] - self._sorted_data[0]
+
+    @property
+    def iqr(self) -> float:
+        return self.q3 - self.q1
+
+    @property
+    def trimean(self) -> float:
+        return 0.25 * (self.q1 + self.q3) + 0.5 * self.median
+
+    @property
+    def lower_cutoff(self) -> float:
+        return self.q1 - 1.5 * self.iqr
+
+    @property
+    def upper_cutoff(self) -> float:
+        return self.q3 + 1.5 * self.iqr
+
+    def draw_histogram(self, bin_size: int = 10, export_path: str = None, **kwargs) -> None:
+        bins = np.arange(self._sorted_data[0] - self.iqr, self._sorted_data[-1] + self.iqr, bin_size)
+        plt.hist(self._data, bins=bins, alpha=0.5)
+        plt.title(f"Histogram (bin_size: {bin_size})")
+        show_or_save(export_path, **kwargs)
+
+    def qq_plot(self, export_path: str = None, **kwargs) -> None:
+        stats.probplot(self._data, dist="norm", plot=plt)
+        show_or_save(export_path, **kwargs)
+
+    def box_plot(self, export_path: str = None, **kwargs) -> None:
+        plt.figure()
+        plt.boxplot(self._data, vert=False, showmeans=True)
+        show_or_save(export_path, **kwargs)
+
+
 __all__ = [
     "Anneal", "Metrics", "ScalarEMA", "Visualizer", "Tracker",
     "Estimator", "ModelPattern", "EnsemblePattern", "Comparer",
     "collate_fn_type", "estimate_fn_type", "scoring_fn_type",
     "generic_data_type", "pattern_type", "patterns_type",
-    "register_metric"
+    "register_metric", "DataInspector",
 ]
